#!/usr/bin/env Rscript

library("neuralnet")

#
# analyze audio files
#

audio_analyze <- function (fname) {
  d<-as.data.frame(read.csv(fname, sep=",", header=TRUE))
  colnames(d) <- c("fname", "bitrate", "format", "filesize", "deltas")
  return(d)
}

audio_plot_bitrate <- function(d, ylimit=0.15) {
  oggs<-subset(d, format=="ogg", c(bitrate,deltas))
  mp3s<-subset(d, format=="mp3", c(bitrate,deltas))
  mp3hs<-subset(d, format=="mp3h", c(bitrate,deltas))
  mp3cs<-subset(d, format=="mp3-cbr", c(bitrate,deltas))
  mp3hus<-subset(d, format=="mp3hu", c(bitrate,deltas))
  aacs<-subset(d, format=="AAC-100", c(bitrate,deltas))
  plot(d$bitrate, d$deltas, ylim=c(0,ylimit))
  points(aacs, col="black", type="l")
  points(mp3s, col="red", type="l")
  points(mp3hs, col="purple", type="l")
  points(mp3hus, col="orange", type="l")
  points(mp3cs, col="blue", type="l")
  points(oggs, col="green", type="l")
}

audio_plot_size <- function(d, ylimit=c(0,1e7)) {
  oggs<-subset(d, format=="ogg", c(bitrate,filesize))
  mp3s<-subset(d, format=="mp3", c(bitrate,filesize))
  mp3hs<-subset(d, format=="mp3h", c(bitrate,filesize))
  mp3cs<-subset(d, format=="mp3-cbr", c(bitrate,filesize))
  mp3hus<-subset(d, format=="mp3hu", c(bitrate,filesize))
  aacs<-subset(d, format=="AAC-100", c(bitrate,filesize))
  plot(d$bitrate, d$filesize, ylim=ylimit)
  points(aacs, col="black", type="l")
  points(mp3s, col="red", type="l")
  points(mp3hs, col="purple", type="l")
  points(mp3hus, col="orange", type="l")
  points(mp3cs, col="blue", type="l")
  points(oggs, col="green", type="l")
}

do_plots <- function() {
  #Plot a generic distribution of everything
  plot(d, cex=0.25)
  #Overview of deltas aggregated by bitrate and format
  audio_plot_bitrate(aggs, ylimit=0.12)
  #Deltas by bitrate and format, high-quality bitrates (>100) only
  audio_plot_bitrate(aggs_hbr, ylimit=0.0071)
  #Is file-size a consideration?
  audio_plot_size(aggs_hbr, ylimit=c(2.2774e7, 2.2778e7))
}

do_pngs <- function() {
  png(width=1024, height=768)
  do_plots()
  dev.off()
}

d <- audio_analyze("sample.csv")
tracknos<-as.numeric(sub("(soundcheck|other-CDs)/([0-9]+).*", "\\2", d$fname))
d$tracknos <- tracknos

#audio_plot(d)
aggs<-aggregate(d, by=list(format=d$format, bitrate=d$bitrate), na.action=na.omit, na.rm=TRUE, FUN=mean)

#nn <- neuralnet(deltas~bitrate,aggs, hidden=100, threshold=0.001)

categories<-read.csv("soundcheck-categories.csv", sep="\t")
d$genres <- categories$genre[ d$tracknos ]
d$polyphony <- categories$polyphony[ d$tracknos ]
d$complexity <- categories$complexity[ d$tracknos ]

cd <- cbind(d$deltas, d$tracknos, d$bitrate, d$format, d$genres, d$filesize, d$complexity)
colnames(cd)<-c("deltas", "trackno", "bitrate", "format", "genre", "filesize", "complexity")

hbr<- d[d$bitrate>100, ]
aggs_hbr<-aggregate(hbr, by=list(format=hbr$format, bitrate=hbr$bitrate), na.action=na.omit, na.rm=TRUE, FUN=mean)
cd_hbr <- cbind(d$deltas, d$tracknos, d$bitrate, d$format, d$genres, d$filesize, d$complexity)
colnames(cd_hbr)<-c("deltas", "trackno", "bitrate", "format", "genre", "filesize", "complexity")

cat("\nContributing factors - deltas:\n")
print(abs(cor(cd_hbr, use="complete.obs")[,1]))

cat("\nContributing factors - filesize:\n")
print(abs(cor(cd_hbr, use="complete.obs")[,6]))

cat("\nAggregates by genre:\n")
print(aggregate(d, by=list(genre=d$genres), na.action=na.omit, na.rm=TRUE, trim=0.1, FUN=mean)[c(1,5,6)])

cat("\nAggregates by genre - high-bitrate:\n")
print(aggregate(hbr, by=list(genre=hbr$genres), na.action=na.omit, na.rm=TRUE, trim=0.1, FUN=mean)[c(1,5,6)])

cat("\nAggregates by format - high-bitrate:\n")
print(aggregate(hbr, by=list(genre=hbr$format), na.action=na.omit, na.rm=TRUE, trim=0.1, FUN=mean)[c(1,5,6)])
